<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 *
 * With the purchase or the installation of the software in your application
 *
 * you accept the licence agreement.
 *
 *
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    Banca Sella S.p.A
 *  @copyright 1996-2024 Banca Sella S.p.A
 *  @license   LICENSE.txt
 */

/* if (!defined('_PS_VERSION_')) {
    exit;
} */

use GuzzleHttp\Client;

class APPpagoValidationModuleFrontController extends ModuleFrontController
{
    private $paymentModule;

    public function __construct($paymentModule)
    {
        $this->paymentModule = $paymentModule;
        parent::__construct();
    }

    public function initContent()
    {
        parent::initContent();
        $this->setTemplate('module:apppago/views/templates/front/payment_options_priv.tpl');
    }

    public function postProcess()
    {
        if (!($this->paymentModule instanceof PaymentModule)) {
            $this->paymentModule = Module::getInstanceByName('apppago');
            if (!($this->paymentModule instanceof PaymentModule)) {
                $this->log('Failed to re-instantiate PaymentModule.', 3);
            }
        }

        if (!Tools::getValue('apppago_cf') || !preg_match("/^[A-Za-z]{6}[0-9]{2}[A-Za-z][0-9]{2}[A-Za-z][0-9]{3}[A-Za-z]$/", Tools::getValue('apppago_cf'))) {
            // add error message in the checkout prestashop page visible after a redirect
            $this->context->cookie->__set('redirect_errors', Tools::displayError('Codice Fiscale non valido'));
            $this->context->cookie->__set('apppago_cf', Tools::getValue('apppago_cf'));

            $this->log('Codice fiscale non valido: ' . Tools::getValue('apppago_cf'), 1);

            Tools::redirect('index.php?controller=order&step=1');
        }

        $this->log('Inizio procedura.', 1);
        // get order id and order details
        $cart = $this->context->cart;

        // Get customer
        $customer = new Customer((int) $cart->id_customer);

        // Get address
        $address = new Address((int) $cart->id_address_invoice);

        $callback_url = $this->context->link->getModuleLink($this->module->name, 'OrderConfirmation', [], true);

        $order_id = $cart->id;

        $olMerchantTrxId = substr(str_shuffle('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'), 0, 5);

        $body = [
            'body' => [
                'olMerchantTrxId' => $olMerchantTrxId,
                'merchantId' => Configuration::get('APPPAGO_MERCHANT_CODE'),
                'loanTotalAmount' => '' . $cart->getOrderTotal(true, Cart::BOTH) . '',
                'installmentNumber' => Configuration::get('APPPAGO_INSTALLMENTS_TYPE') ? Configuration::get('APPPAGO_FIXED_INSTALLMENTS_NUMBER') : '',
                'taxCode' => strtoupper(Tools::getValue('apppago_cf')) ?? '',
                'merchantRedirect' => $callback_url,
                'firstname' => $customer->firstname,
                'lastname' => $customer->lastname,
                'residenceAddress' => $address->address1 . ' ' . $address->address2,
                'residenceCity' => $address->city,
                'residenceState' => State::getNameById($address->id_state),
                'residencePostalCode' => $address->postcode,
                'email' => $customer->email,
                'phone' => $address->phone ?? $address->phone_mobile ?? '',
                'merchant_order_id' => '' . $order_id . '',
            ],
        ];

        $apiUrl = 'https://secure.sellapersonalcredit.it/SPCPlugAppPago/api/order';

        if (Configuration::get('APPPAGO_TESTING_MODE') == 'enabled') {
            $apiUrl = 'https://sandbox.sellapersonalcredit.it/SPCPlugAppPago/api/order';
        }

        $this->log('Url chiamata: ' . $apiUrl, 1);
        $this->log('AppPago - Body chiamata: ' . json_encode($body), 1);

        try {
            // ATTENZIONE: mantenere sia l'opzione http_errors che il controllo della risposta per versioni differenti di Guzzle in prestashop 1.7 e 8
            $client = new Client([
                'http_errors' => false
            ]);
            $options = [
                'headers' => [
                    'Content-Type' => 'application/json',
                    'Accept' => 'application/json'
                ],
                //'debug' => true,
                'body' => json_encode($body['body'])  // Se hai bisogno di inviare una stringa JSON
            ];
            $response = $client->post($apiUrl, $options);
            $data = json_decode($response->getBody(), true);
            $this->log('AppPago - Risposta: ' . json_encode($data), 1);

            if (isset($data['redirectUrl'])) {
                // create order
                $this->paymentModule->validateOrder(
                    // fake error here, see first if statement above
                    $cart->id,
                    Configuration::get('APPPAGO_WAITING_STATE'),
                    $cart->getOrderTotal(true, Cart::BOTH),
                    $this->module->displayName,
                    null,
                    [],
                    null,
                    false,
                    $customer->secure_key
                );

                Tools::redirect($data['redirectUrl']);
            } else {
                $error = isset($data['error']) ? $data['error'] : "AppPago errore sconosciuto. Riprovare più tardi.";
                $this->context->cookie->__set('redirect_errors', Tools::displayError($error));
                Tools::redirect('index.php?controller=order&step=1');
            }
        } catch (GuzzleHttp\Exception\ClientException $e) {
            $response = $e->getResponse();
            $data = json_decode($response->getBody(), true);
            $error = isset($data['error']) ? $data['error'] : "AppPAgo errore sconosciuto, riprovare più tardi.";
            $this->log('CErrore: ' . var_export($e->getTraceAsString(), true), 1);
            $this->context->cookie->__set('redirect_errors', Tools::displayError($error));
            Tools::redirect('index.php?controller=order&step=1');
        } catch (Exception $e) {
            $this->log('Errore: ' . $e->getTraceAsString(), 1);
            $this->context->cookie->__set('redirect_errors', Tools::displayError('Errore sconosciuto. Riprovare più tardi.'));
            Tools::redirect('index.php?controller=order&step=1');
        }
    }

    public function log($message, $severity = 1)
    {
        if (defined('_PS_MODE_DEV_') && _PS_MODE_DEV_) {
            PrestaShopLogger::addLog('AppPago - ' . $message, $severity);
        }
    }
}
